"use strict";
// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoggerNodeHandler = exports.colorString = void 0;
const tslib_1 = require("tslib");
const log_js_1 = require("../common/log.js");
const time_js_1 = require("../common/time.js");
const tty_1 = tslib_1.__importDefault(require("tty"));
const log_filter_js_1 = require("../common/log-filter.js");
const convert_js_1 = require("../common/convert.js");
const log_util_js_1 = require("../common/log-util.js");
const colors = [6, 2, 3, 4, 5, 1];
function selectColor(namespace) {
    let hash = 0;
    for (let i = 0; i < namespace.length; i++) {
        hash = (hash << 5) - hash + namespace.charCodeAt(i);
        hash |= 0; // Convert to 32bit integer
    }
    return colors[Math.abs(hash) % colors.length];
}
let namespaces = {};
let time = time_js_1.getTimestamp();
const useColors = tty_1.default.isatty(process.stderr.fd);
function log(...args) {
    return process.stderr.write(convert_js_1.renderMessages(args) + "\n");
}
// export const BOLD = Symbol()
// export const UNBOLD = Symbol()
// export const BLUE = Symbol()
// export const GREY = Symbol()
// export const GREEN = Symbol()
// export const RED = Symbol()
// export const PURPLE = Symbol()
// export const ORANGE = Symbol()
// export const UNCOLOR = Symbol()
// const _browserStyleMap = {
//   [BOLD]: pair.create("font-weight", "bold"),
//   [UNBOLD]: pair.create("font-weight", "normal"),
//   [BLUE]: pair.create("color", "blue"),
//   [GREEN]: pair.create("color", "green"),
//   [GREY]: pair.create("color", "grey"),
//   [RED]: pair.create("color", "red"),
//   [PURPLE]: pair.create("color", "purple"),
//   [ORANGE]: pair.create("color", "orange"), // not well supported in chrome when debugging node with inspector - TODO: deprecate
//   [UNCOLOR]: pair.create("color", "black"),
// }
// const _nodeStyleMap = {
//   [BOLD]: "\u001b[1m",
//   [UNBOLD]: "\u001b[2m",
//   [BLUE]: "\x1b[34m",
//   [GREEN]: "\x1b[32m",
//   [GREY]: "\u001b[37m",
//   [RED]: "\x1b[31m",
//   [PURPLE]: "\x1b[35m",
//   [ORANGE]: "\x1b[38;5;208m",
//   [UNCOLOR]: "\x1b[0m",
// }
var COLOR;
(function (COLOR) {
    COLOR[COLOR["RED"] = 1] = "RED";
    COLOR[COLOR["GREEN"] = 2] = "GREEN";
    COLOR[COLOR["BLUE"] = 4] = "BLUE";
    COLOR[COLOR["PURPLE"] = 5] = "PURPLE";
    COLOR[COLOR["GRAY"] = 7] = "GRAY";
    COLOR[COLOR["ORANGE"] = 8] = "ORANGE";
})(COLOR || (COLOR = {}));
const colorEnd = "\u001B[0m";
function colorString(value, colorCode) {
    const colorStart = "\u001B[3" + (colorCode < 8 ? colorCode : "8;5;" + colorCode) + "m";
    return `${colorStart}${value}${colorEnd}`;
}
exports.colorString = colorString;
function LoggerNodeHandler(opt = {}) {
    var _a;
    const { level = log_js_1.LogLevel.all, colors = true, levelHelper = true, nameBrackets = true, padding = 0, filter = undefined, stack = true, } = opt;
    const matches = log_filter_js_1.useNamespaceFilter((_a = filter !== null && filter !== void 0 ? filter : process.env.ZEED) !== null && _a !== void 0 ? _a : process.env.DEBUG);
    return (msg) => {
        if (msg.level < level)
            return;
        if (!matches(msg.name))
            return;
        const timeNow = time_js_1.getTimestamp();
        let name = msg.name || "";
        let ninfo = namespaces[name || ""];
        if (ninfo == null) {
            ninfo = {
                color: selectColor(name),
                // time: timeNow
            };
            namespaces[name] = ninfo;
        }
        const diff = time_js_1.formatMilliseconds(timeNow - time);
        let args;
        let displayName = nameBrackets ? `[${name}]` : name;
        if (padding > 0) {
            displayName = displayName.padStart(padding, " ");
            // displayName = displayName.padEnd(padding, " ")
        }
        if (colors && useColors) {
            const c = ninfo.color;
            args = [colorString(displayName, c) + ` | `]; // nameBrackets ? [`%c[${name}]`] : [`%c${name}`]
            args.push(...msg.messages);
            args.push(colorString(`+${diff}`, c));
        }
        else {
            args = [displayName, ...msg.messages];
            args.push(`+${diff}`);
        }
        if (stack) {
            const line = log_util_js_1.getSourceLocation(4, true);
            if (line) {
                args.push(colorString(`(${line})`, COLOR.GRAY));
            }
        }
        switch (msg.level) {
            case log_js_1.LogLevel.info:
                if (levelHelper)
                    args[0] = `I|*   ` + args[0];
                log(...args);
                break;
            case log_js_1.LogLevel.warn:
                if (levelHelper)
                    args[0] = `W|**  ` + args[0];
                log(...args);
                break;
            case log_js_1.LogLevel.error:
                if (levelHelper)
                    args[0] = `E|*** ` + args[0];
                log(...args);
                break;
            default:
                if (levelHelper)
                    args[0] = `D|    ` + args[0];
                log(...args);
                break;
        }
    };
}
exports.LoggerNodeHandler = LoggerNodeHandler;
//# sourceMappingURL=log-node.js.map