// (C)opyright 2021-07-15 Dirk Holtwick, holtwick.it. All rights reserved.
import { readFileSync, writeFileSync, mkdirSync, rmSync, unlinkSync, readdirSync, } from "fs";
import { resolve } from "path";
import { Logger } from "../common/log.js";
const log = Logger("zeed:filestorage");
export class FileStorage {
    constructor(opt = {}) {
        var _a, _b, _c;
        this.store = {};
        this.fileKeys = undefined;
        this.pretty = false;
        this.dirname = resolve(process.cwd(), opt.path || ".fileStorage");
        this.pretty = !!opt.pretty;
        this.extension = (_a = opt.extension) !== null && _a !== void 0 ? _a : ".json";
        if (!this.extension.startsWith("."))
            this.extension = "." + this.extension;
        this.extensionLength = this.extension.length;
        this.objectToString =
            (_b = opt.objectToString) !== null && _b !== void 0 ? _b : ((data) => {
                return this.pretty
                    ? JSON.stringify(data, null, 2)
                    : JSON.stringify(data);
            });
        this.objectFromString =
            (_c = opt.objectFromString) !== null && _c !== void 0 ? _c : ((data) => {
                try {
                    return JSON.parse(data);
                }
                catch (err) {
                    log.warn(`fileStorage parse error '${err}' in`, data);
                }
            });
    }
    setItem(key, value) {
        const data = this.objectToString(value);
        this.store[key] = data;
        try {
            const path = resolve(this.dirname, key + this.extension);
            mkdirSync(this.dirname, { recursive: true });
            writeFileSync(path, data, "utf8");
        }
        catch (err) {
            log.error("setItem error", err);
        }
    }
    getPath(key) {
        return resolve(this.dirname, key + this.extension);
    }
    getBuffer(key) {
        const path = resolve(this.dirname, key + this.extension);
        return Buffer.from(readFileSync(path));
    }
    getItem(key) {
        if (this.store.hasOwnProperty(key)) {
            let value = this.store[key];
            return this.objectFromString(value) || null;
        }
        else {
            try {
                const path = resolve(this.dirname, key + this.extension);
                const data = readFileSync(path, "utf8");
                if (data) {
                    return this.objectFromString(data);
                }
            }
            catch (err) {
                log.error("getItem error", err);
            }
        }
        return null;
    }
    removeItem(key) {
        delete this.store[key];
        if (this.fileKeys != null) {
            const index = this.fileKeys.indexOf(key);
            if (index !== -1) {
                this.fileKeys.splice(index, 1);
            }
        }
        try {
            const path = resolve(this.dirname, key + this.extension);
            unlinkSync(path);
        }
        catch (err) { }
    }
    clear() {
        this.fileKeys = [];
        this.store = {};
        rmSync(this.dirname, { recursive: true, force: true });
    }
    allKeys() {
        if (this.fileKeys == null) {
            try {
                this.fileKeys =
                    readdirSync(this.dirname, { withFileTypes: true })
                        .filter((item) => !item.isDirectory() && item.name.endsWith(this.extension))
                        .map((item) => item.name.slice(0, -this.extensionLength)) || [];
            }
            catch (err) { }
        }
        let keys = [...(this.fileKeys || [])];
        for (let key of Object.keys(this.store)) {
            if (!keys.includes(key)) {
                keys.push(key);
            }
        }
        keys.sort();
        return keys;
    }
}
//# sourceMappingURL=filestorage.js.map